#include <sys/types.h>  /* for open()ing /dev/random */
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>  /* for read()ing /dev/random */

#include <rfftw.h>
#include <math.h>  /* for M_PI, atan2() */

#define N (1000)

int
main(int argc, char **argv)
{
     fftw_real in[N], out[N];
     fftw_real power_spectrum[N/2+1], phase[N/2+1];
     rfftw_plan p;
     int k;
     fftw_real x;

     p = rfftw_create_plan(N, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);

     read(open("/dev/random", O_RDONLY), &k, sizeof(k));
     srand(k);

     /* Fill 'in' table. */
     for (k=0; k<N; k++) {
       x = (fftw_real)k / (M_PI/2.0) /**- M_E**/;
       in[k] = sin(x) + sin(3*x)/3.0 + sin(5*x)
#if 1
	 /* Noise. */
	 + (fftw_real)rand() / (fftw_real)RAND_MAX / 10.0
#endif
	 ;
#if 0
       /* One-bit sampling. */
       if (in[k] > 0.0) {
	 in[k] = +1.0;
       } else {
	 in[k] = -1.0;
       }
#endif
#if 1
       /* Two-bit sampling. */
       if (in[k] > 1.0) {
	 in[k] = +1.0;
       } else if (in[k] > 0.5) {
	 in[k] = +0.5;
       } else if (in[k] > -0.5) {
	 in[k] = -0.5;
       } else {
	 in[k] = -1.0;
       }
#endif
     }

     /* FFT. */
     rfftw_one(p, in, out);
     power_spectrum[0] = out[0]*out[0];  /* DC component */
     phase[0] = 0.0;
     for (k = 1; k < (N+1)/2; ++k) {  /* (k < N/2 rounded up) */
          power_spectrum[k] = out[k]*out[k] + out[N-k]*out[N-k];
	  phase[k] = atan2(out[N-k], out[k])*360.0/M_PI;
     }
     if (N % 2 == 0) {  /* N is even */
          power_spectrum[N/2] = out[N/2]*out[N/2];  /* Nyquist freq. */
	  phase[N/2] = 0.0;
     }

     /* Output spectrum. */
     for (k=0; k<(N/2); k++) {
       printf("%f\t%f\n", power_spectrum[k], phase[k]);
     }

     rfftw_destroy_plan(p);

     return (EXIT_SUCCESS);
}  /* main */
