/* mrg.c -- Merge files from multiple VSIB computers, */
/*          interleaving 32/16/8 bit words. */

/*
 * $Log: mrg.c,v $
 * Revision 1.1  2002/09/20 07:39:15  amn
 * Added mrg.c, merge/split for multi-PC files.
 *
 *
 */

/* Copyright (C) 2001--2002 Ari Mujunen, Ari.Mujunen@hut.fi

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <sys/types.h>  /* open() */
#include <sys/stat.h>  /* open() */
#include <fcntl.h>  /* open() */
#include <unistd.h>  /* read(), write(), close() */
#include <string.h>  /* strstr() */


typedef struct sDir {
  char *nameTemplate;
  int nameNumber;
  int blocksPerFile;
  int fid;
  int nowBlocks;
} tDir, *pDir;



int
main(int argc, char *argv[]) {
  int readMode;
  int blksize;
  int bytesinword;
  int totalblks;
  int memblks;
  char *memblk;
  char *convblk;
  char *p;
  int ndirs;
  pDir dirs;
  int i;
  int blk;


  if (argc < 4) {
    fprintf(stderr, "%s: needs at least blocksize bytes_in_word totalblocks ndirs { path%%d blks }\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  readMode = !(strstr(argv[0], "spl") != NULL);
  blksize = atoi(argv[1]);
  bytesinword = atoi(argv[2]);
  assert( (bytesinword==4)||(bytesinword==2)||(bytesinword==1) );
  assert( (blksize % bytesinword) == 0 );
  totalblks = atoi(argv[3]);
  ndirs = atoi(argv[4]);
  memblks = ndirs;
  assert( (memblk = malloc(memblks*blksize)) != NULL );
  assert( (convblk = malloc(memblks*blksize)) != NULL );
  assert( (argc >= (5 + ndirs*2)) );
  assert( (dirs = malloc((ndirs)*sizeof(tDir))) != NULL );
  for (i=0; i<ndirs; i++) {
    dirs[i].nameTemplate = argv[5+i*2];
    dirs[i].nameNumber = 0;
    dirs[i].blocksPerFile = atoi(argv[6+i*2]);
    dirs[i].nowBlocks = dirs[i].blocksPerFile;  /* init to end */
  }  /* for each directory */

  /* Start looping for total number of blocks. */
  for (blk=0; blk<totalblks; blk++) {

    /* Which file of the multiple in/out files. */
    i = blk % ndirs;

    /* Splitting: get one large 'convblk' and split into 'memblk'. */
    if ((!readMode) && (i == 0)) {
      /* Read one large block. */
      assert( (read (STDIN_FILENO, convblk, ndirs*blksize)) == (ndirs*blksize) );
      if (bytesinword == 4) {
#define SPLITCOPY(datatype) \
        datatype *s, *t[ndirs];  /* source, target */ \
        int j; \
         \
        /* Set up the starting word pointers. */ \
        for (j=0; j<ndirs; j++) { \
          t[j] = (datatype *)(&(memblk[(j)*blksize])); \
        } \
        s = (datatype *)convblk; \
         \
        /* For every word in output blocks... */ \
        for (j=0; j<(blksize/sizeof(datatype)); j++) { \
          int k; \
           \
          /* ...copy words from a "bunch" in input block to output block. */ \
          for (k=0; k<ndirs; k++) { \
            *(t[k])++ = *s++; \
          }  /* for k */ \
        }  /* for every word in block */
        SPLITCOPY(unsigned long) 
      } else if (bytesinword == 2) {
        SPLITCOPY(unsigned short) 
      } else if (bytesinword == 1) {
        SPLITCOPY(unsigned char) 
      } else {
        fprintf(stderr, "%s: bytes_in_word not 4/2/1\n", argv[0]);
        exit (EXIT_FAILURE);
      }
    }  /* if before first block of one 'ndirs' "row" */

    /* Handle multiple files. */
    if (dirs[i].nowBlocks >= dirs[i].blocksPerFile) {
      /* Open a new file in this directory / template. */
      char pn[255];
      int n = blk / dirs[i].blocksPerFile;

      snprintf(pn, sizeof(pn), dirs[i].nameTemplate, n);
      if (readMode) {
        assert( (dirs[i].fid = open(pn, O_RDONLY)) != -1 );
      } else {
        assert( (dirs[i].fid = creat(pn, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) != -1 );
      }
      fprintf(stderr, "at block = %d, opened file '%s'\n", blk, pn);
      dirs[i].nowBlocks = 0;
    }

    /* Write (or read) one block. */
    p = &(memblk[(blk % memblks)*blksize]);
    if (readMode) {

      assert( (read(dirs[i].fid,
                     p,
                     blksize)) == blksize );

    } else {

      assert( (write(dirs[i].fid,
                     p,
                     blksize)) == blksize );
    }
    dirs[i].nowBlocks++;

    /* Time to switch to another file? */
    if (dirs[i].nowBlocks >= dirs[i].blocksPerFile) {
      /* Close the ready file before opening a new. */
      assert( (close(dirs[i].fid) != -1) );
    }

    /* Merging: see if we have a complete 'ndirs' set of blocks in 'memblk'. */
    /* In this case we merge/interleave input blocks to output. */
    if (readMode && (i >= (ndirs-1))) {
      if (bytesinword == 4) {
#define MERGECOPY(datatype) \
        datatype *s[ndirs], *t;  /* source, target */ \
        int j; \
         \
        /* Set up the starting word pointers. */ \
        for (j=0; j<ndirs; j++) { \
          s[j] = (datatype *)(&(memblk[(j)*blksize])); \
        } \
        t = (datatype *)convblk; \
         \
        /* For every word in input blocks... */ \
        for (j=0; j<(blksize/sizeof(datatype)); j++) { \
          int k; \
           \
          /* ...copy a word from each input block to output block. */ \
          for (k=0; k<ndirs; k++) { \
            *t++ = *(s[k])++; \
          }  /* for k */ \
        }  /* for every word in block */
        MERGECOPY(unsigned long) 
      } else if (bytesinword == 2) {
        MERGECOPY(unsigned short) 
      } else if (bytesinword == 1) {
        MERGECOPY(unsigned char) 
      } else {
        fprintf(stderr, "%s: bytes_in_word not 4/2/1\n", argv[0]);
        exit (EXIT_FAILURE);
      }

      /* Write out the block. */
      assert( (write (STDOUT_FILENO, convblk, ndirs*blksize)) == (ndirs*blksize) );
    }  /* if last block of one 'ndirs' "row" */
  }  /* for each block */

  return(EXIT_SUCCESS);
}  /* main */
