#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <sys/ipc.h>  /* for shared memory */
#include <sys/shm.h>

#define MY_NAME "skip: "

/* Upcoming wrshm.h: */
#define fourCharLong(a,b,c,d) ( ((long)a)<<24 | ((long)b)<<16 | ((long)c)<<8 | d )
typedef struct sSh {
  int relSeekBlocks;
} tSh, *ptSh;



int
main(int argc, char *argv[]) {
  /* Shared memory unique identifier (key) and the returned reference id. */
  key_t shKey;
  int shId = -1;
  ptSh sh;

  /* Find and attach to 'wr<-->control' shared memory. */
  shKey = fourCharLong('v','s','i','b');
  /* Don't create, must exist (created by 'wr'). */
  shId = shmget(shKey, sizeof(tSh), 0777);
  if (shId == -1) {
    perror(MY_NAME "shmget(): cannot find 'wr/rd' shared memory");
    exit(EXIT_FAILURE);
  }
  assert( (sh = (ptSh)shmat(shId, NULL, 0)) != (ptSh)-1 );

  /* ----- */
  /* Check if previous seek has been done. */
  if (sh->relSeekBlocks) {
    fprintf(stderr, MY_NAME "previous skip still pending (%+d blocks)\n",
	    sh->relSeekBlocks
	    );
  } else {
    /* Send 'wr' the number (+/-) of blocks it has to seek/skip/replay. */
    if (argc == 2) {
      sh->relSeekBlocks = atoi(argv[1]);
    } else if (argc == 4) {
      int framespersec, sec, sec32m;
      double frames;

      framespersec =  atoi(argv[1]);
      sec = strtol(argv[2], NULL, 0);
      sec32m = strtol(argv[3], NULL, 0);
      frames = (double)framespersec * ((double)sec + ((double)sec32m / 32000000.0));
      fprintf(stderr, MY_NAME "skipping %+d frames\n", (int)frames);
      sh->relSeekBlocks = (int)frames;
    } else {
      fprintf(stderr, MY_NAME "usage: frames [sec 1/32msec]\n");
    }
  }
  /* ----- */

  if ((shId != -1) && (sh != (ptSh)-1) && (sh != NULL)) {
    // It is not our business to remove the seqment,
    // 'wr' takes care of managing removal.
    // assert( shmctl(shId, IPC_RMID, NULL) == 0 );
    assert( shmdt(sh) == 0 );
  }  // if shared memory was attached to

  exit (EXIT_SUCCESS);
} /* main */
