/* vsibcon.cc  --  Controlling the VSIB board and r/w of data files.  */

// $Log$


#include "registration.h"
RCSID(vsibcon_cc, "$Id$");


// Configuration/debugging defines.
#define XYZ1 1

// Potential parameters, now constants.
#define XYZ2 1

// Constants.
#define MYNAME "vsibcon: "


#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <limits.h>
#include <math.h>
#include <time.h>
#include <string.h>

// For shared memory.
#include <sys/ipc.h>
#include <sys/shm.h>

#if 0
/* The direct I/O port reference macros reside here. */
/* They work _only_ if the source files using them are compiled */
/* with '-O' optimization flag on; they rely on 'inline' code generation. */
// Used to be: #include <asm/io.h>
// Now in Debian slink 2.1:
// (Disable 'volatile' in conjunction with 'asm' to get rid of extra warnings
//  which appear with g++ egcs-2.91.60:)
#define __volatile__
#include <sys/io.h>
#endif


/* -------------------------------------------------------------------- */
// Serial port for VSI-S serial line (niy :-)
#include "cSerialLine.hh"

#define SERIAL_PORT_NAME "/dev/ttyS1"
#define baudRate 115200L
// #define baudRate 4800L

class cSerialLine sl;


/*------------------------------------------------------------*/
// The common shared struct
// across vsibcon+vsibcli.
//
#include "sh.h"

// Global shared struct,
// and a pointer to it, for shared memory references in Linux with 'sh->'.
ptSh sh = NULL;



// Timestamp ASCII formatting for log files.
#include "tstamp.c"



/* ------------------------------------------------------------------ */
int main(int argc, char **argv)
{
  // (xxx: Note: classes cannot be easily put into shared struct 'sh->'
  // since their constructor functions won't probably be run.)

  long int vSerialReadErrors = 0;

  // Shared memory unique identifier (key) and the returned reference id.
  key_t shKey;
  int shId = -1;

  // Create and initialize 'vsibcon--vsibcli--xvsibcon' shared memory.
  shKey = fourCharLong('v','s','i','b');
  assert( (shId = shmget(shKey, sizeof(tSh), IPC_CREAT | 0777)) != -1 );
  assert( (sh = (ptSh)shmat(shId, NULL, 0)) != (void *)-1 );
  sh->utc = 0.0;

  // The default we assume 'vsibcli' wants us to do is
  // VAX tracking; however, we do start up in 'standby' mode.
  // (This means "secret YES" button from panel will VAX track.)
  sh->vsibcliUpdates = 0;  // incremented by 'vsibcli', decr by 'vsibcon'
  sh->vsibcliMode = ACM_VAX_track;

  // Init logging to disabled state.
  sh->log.isActive = 0;

  /* "Use" argc, argv to prevent compiler warning messages. */
  (void)argc; (void)argv;


  /* Serial up/downlink. */
  sl.open(SERIAL_PORT_NAME);
  // Can later use 'sl.whatPathname()' to access the stored serial device path.
  sl.serialSettings(/*baudRate=>*/ baudRate,
		    /*numberOfDataBits=>*/ 8,
		    /*numberOfStopBits=>*/ 1,
		    /*whichParity=>*/ eParityNone,
		    /*minBytes=>*/ 0,
		    /*maxTimeout=>*/ 2);  // 0.2 seconds
  // /*maxTimeout=>*/ 5);  // 0.5 seconds

  printf("$Id$\n");
  // The main event processing loop.
  for (;;) {

    if (sh->vsibcliUpdates > 0) {
      // What 'vsibcli' wants us to do?
      switch (sh->vsibcliMode) {
        // Does 'vsibcli' want a standby stop?
        case ACM_standby: {
          vPanel.forceState(/*sw=>*/ SW_standby, sh->interlock,
                            /*pos=>*/ sh->incr,
                            /*off=>*/ sh->panelOffset);
          break;
        }
        // Does 'vsibcli' want a fixed az-el position?
        case ACM_azel: 
        case ACM_VAX_track: {
          // Start tracking az-el.
          vPanel.forceState(/*sw=>*/ SW_yes_ok, sh->interlock,
                            /*pos=>*/ sh->incr,
                            /*off=>*/ sh->panelOffset);
          break;
        }
        // Does 'vsibcli' want source changed?
        case ACM_source: {
          if (sh->isPlanet) {
            // Is a solar system planet.
            vNovas.setPlanet(sh->sourcePlanetNumber);
            // xxx: maybe one 'updateApparent()' to get the following:
            // xxx: copy calculated ra, dec, and today's epoch
            //      to 'sh->source*'
          } else {
            // Is a regular object.
            vNovas.setSource(sh->sourceName,
                             sh->sourceRa,
                             sh->sourceDec,
                             sh->sourceEpoch,
                             sh->sourceRadialvel,
                             sh->sourcePromora,
                             sh->sourcePromodec,
                             sh->sourceParallax);
          }
          // Start tracking this source.
          vPanel.forceState(/*sw=>*/ SW_yes_ok, sh->interlock,
                            /*pos=>*/ sh->incr,
                            /*off=>*/ sh->panelOffset);
          break;
        }
        default: {
          // xxx: nothing, no known 'sh->vsibcliMode'
          break;
        }
      }  // switch
      sh->vsibcliUpdates--;
    }  // if updates

    // Get the time when to calculate the coordinates.
    assert( gettimeofday(&(sh->coord_calc_utc_tv), /*tz=>*/ NULL) == 0 );
    sh->utc = dsecs(sh->coord_calc_utc_tv);
    // xxx: other time calculations to 'sh->' (and to display)?
    // sh->utcMJD ???
    // sh->DOY ???


lEnd:
  /* Cleanups. */
  sl.close();

#if 0
  printf("shKey=0x%08X, shId=%d, sh=%08X\n",
         shKey, shId, (unsigned int)sh);
#endif

  // Detach from shared memory (but don't delete it).
  if ((shId != -1) && (sh != (ptSh)-1) && (sh != NULL)) {
    assert( shmctl(shId, IPC_RMID, NULL) == 0 );
    assert( shmdt(sh) == 0 );
  }  // if shared memory was allocated

  return (EXIT_SUCCESS);
}  // main
